<?php

class Admin extends Model {

    public static function estAdmin($slug) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_admin WHERE identite = :slug"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(":slug" => $slug));
        return ($req->rowcount() != 0);
    }

    public static function getListeJeuNonValide() {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_jeu WHERE NOT est_valide ORDER BY nom"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute();
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne;
        }
        return $res;
    }

    public static function acceptJeu($slug) {
        $db = Database::getInstance();
        $sql = "UPDATE R_jeu SET est_valide = 1 WHERE slug = :slug";
        $req = $db->prepare($sql);
        return $req->execute(array(':slug' => $slug));
    }

    public static function supprJeu($slug) {
        $db = Database::getInstance();
        $sql= "SELECT * FROM R_partie WHERE jeu = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql= "DELETE FROM R_joueur_partie WHERE partie = :partie";
        $req = $db->prepare($sql);
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) {
            $req->execute(array(':partie' => $ligne['id']));
        }
        $sql= "DELETE FROM R_partie WHERE jeu = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql= "DELETE FROM R_joueur_jeu WHERE jeu = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql = "DELETE FROM R_jeu WHERE slug = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
    }

    public static function getListeJeuValide() {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_jeu WHERE est_valide ORDER BY nom"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute();
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne;
        }
        return $res;
    }
    
    public static function unacceptJeu($slug) {
        $db = Database::getInstance();
        $sql = "UPDATE R_jeu SET est_valide = 0 WHERE slug = :slug";
        $req = $db->prepare($sql);
        return $req->execute(array(':slug' => $slug));
    }
    
    public static function setModifJeu($slug, $data) {
        $db = Database::getInstance();
        $sql = "UPDATE R_jeu SET description = :description, max_joueurs = :max_joueurs,
                min_joueurs = :min_joueurs, max_equipes = :max_equipes, min_equipes = :min_equipes,
                 type = :type, type_scores = :type_scores WHERE slug = :slug";
        $req = $db->prepare($sql);
        return $req->execute(array(':slug' => $slug,
                                    ':description' => $data['description'] ,
                                    ':type' => $data['type'],
                                    ':max_joueurs' => $data['max_joueurs'],
                                    ':min_joueurs' => $data['min_joueurs'],
                                    ':max_equipes' => $data['max_equipes'],
                                    ':min_equipes' => $data['min_equipes'],
                                    ':type_scores' => $data['type_scores']));
    }

    public static function supprJoueur($slug) {
        $db = Database::getInstance();
        $sql= "DELETE FROM R_joueur_partie WHERE joueur = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql= "DELETE FROM R_joueur_jeu WHERE joueur = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
        $sql = "DELETE FROM R_joueur WHERE slug = :slug";
        $req = $db->prepare($sql);
        $req->execute(array(':slug' => $slug));
    }

    public static function getListPartie() {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_partie JOIN R_jeu ON R_partie.jeu = R_jeu.slug ORDER BY date"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute();
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne;
        }
        return $res;
    }

    public static function getEquipes($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        $i = -1;
        $classement_old = 0;
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) {
            if($classement_old != $ligne['classement']) {
                $i = $i +1;
                $classement_old = $ligne['classement'];
                $res[$i] = array();
            }
            $res[$i][] = Profil::getFromSlug($ligne['joueur']);
        }
        return $res;
    }

    public static function getListeJoueur($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = Profil::getFromSlug($ligne['joueur']);
        }
        return $res;
    }

    public static function getListeScore($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[$ligne['joueur']] = $ligne['score'];
        }
        return $res;
    }

    public static function getListeScorePartie($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[$ligne['joueur']] = $ligne['score_partie'];
        }
        return $res;
    }

    public static function getListeScorePartie2($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne['score_partie'];
        }
        return $res;
    }

    public static function getListeClassement($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT * FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[$ligne['joueur']] = $ligne['classement'];
        }
        return $res;
    }

    public static function getListeClassement2($id) {
        $db = Database::getInstance(); // on récupère la connexion à la BDD
        $sql = "SELECT DISTINCT classement FROM R_joueur_partie WHERE partie = :id ORDER BY classement"; // on définit la requête qu'on enverra (on considère que le slug contient jeu/nomDuJeu pour le classement par jeu. Donc il faudrait encore faire un explode() sur le slug.
        $req = $db->prepare($sql); // on prépare la requête (c'est là que la mémthode __call() est utile dans la classe Database d'ailleurs)
        $req->execute(array(':id' => $id));
        $res = array();
        while($ligne = $req->fetch(PDO::FETCH_ASSOC)) { // parcours toute la liste
            $res[] = $ligne['classement'];
        }
        return $res;
    }

    public static function ModifPartie($id, $joueurs, $scores, $jeu)
    {
        $db = Database::getInstance();
        $sql5 = "UPDATE R_joueur_jeu SET score = :new_score WHERE joueur = :id_joueur AND jeu = :id_jeu;";
        $stmt5 = $db->prepare($sql5);

        $partieModif = Partie::getFromId($id);
        // recherche des joueurs ayant fais une partie apres celle-ci (celle-ci incluse)
        $sql1 = "SELECT * FROM R_joueur_partie JOIN R_partie ON R_joueur_partie.partie = R_partie.id WHERE date >= date(:date) AND jeu =:jeu ORDER BY date";
        $stmt1 = $db->prepare($sql1);
        $stmt1->execute(array(':date' => $partieModif->date, ":jeu" => $partieModif->jeu));
        // pour tout les joueurs remettre les scrores comme-ci aucune partie après celle-ci a été jouée
        $joueurDejaTraite = array();
        while ($joueur = $stmt1->fetch(PDO::FETCH_ASSOC)) {
            if (!in_array($joueur['joueur'], $joueurDejaTraite)) {
                $joueurDejaTraite[] = $joueur['joueur'];
                $stmt5->execute(array(':new_score' => $joueur['score'],
                ':id_joueur' => $joueur['joueur'],
                    ':id_jeu' => $joueur['jeu']));
            }
        }
        // applique la partie
        if (($resp = Partie::modPartie($joueurs, $scores, $jeu, $id)) > 0){
            print $resp;
            die();
        }
        // pour chaque partie apres celle modifier on la réapplique avec les nouveaux scores
        $sql2 = "SELECT * FROM R_partie WHERE date > date(:date) AND jeu =:jeu ORDER BY date";
        $stmt2 = $db->prepare($sql2);
        $stmt2->execute(array(':date' => $partieModif->date, ":jeu" => $partieModif->jeu));
        while ($ligne = $stmt2->fetch(PDO::FETCH_ASSOC)) {
            $joueurs = $jeu->type == 'equipes' ? Admin::getEquipes($ligne['id']) : Admin::getListeJoueur($ligne['id']);
            $scores = $jeu->type_scores == 'scores' ? Admin::getListeScorePartie2($ligne['id']) : Admin::getListeClassement2($ligne['id']);
            Partie::modPartie($joueurs, $scores, $jeu, $ligne['id']);
        }
        return 0;
    }

    public static function supprPartie($id) {
        $db = Database::getInstance();
        $sql5 = "UPDATE R_joueur_jeu SET score = :new_score WHERE joueur = :id_joueur AND jeu = :id_jeu;";
        $stmt5 = $db->prepare($sql5);
        $partieSuppr = Partie::getFromId($id);
        // recherche des joueurs ayant fais une partie apres celle-ci (celle-ci incluse)
        $sql1 = "SELECT * FROM R_joueur_partie JOIN R_partie ON R_joueur_partie.partie = R_partie.id WHERE date >= date(:date) AND jeu =:jeu ORDER BY date";
        $stmt1 = $db->prepare($sql1);
        $stmt1->execute(array(':date' => $partieSuppr->date, ":jeu" => $partieSuppr->jeu));
        // pour tout les joueurs remettre les scrores comme-ci aucune partie après celle-ci a été jouée
        $joueurDejaTraite = array();
        while ($joueur = $stmt1->fetch(PDO::FETCH_ASSOC)) {
            if (!in_array($joueur['joueur'], $joueurDejaTraite)) {
                $joueurDejaTraite[] = $joueur['joueur'];
                $stmt5->execute(array(':new_score' => $joueur['score'],
                ':id_joueur' => $joueur['joueur'],
                    ':id_jeu' => $joueur['jeu']));
            }
        }

        $sql = "DELETE FROM R_joueur_partie WHERE partie = :id";
        $req = $db->prepare($sql);
        $req->execute(array(':id' => $id));

        $sql = "DELETE FROM R_partie WHERE id = :id";
        $req = $db->prepare($sql);
        $req->execute(array(':id' => $id));

        $jeu = Jeu::getFromSlug($partieSuppr->jeu);
        $sql2 = "SELECT * FROM R_partie WHERE date > date(:date) AND jeu =:jeu ORDER BY date";
        $stmt2 = $db->prepare($sql2);
        $stmt2->execute(array(':date' => $partieSuppr->date, ":jeu" => $partieSuppr->jeu));
        while ($ligne = $stmt2->fetch(PDO::FETCH_ASSOC)) {
            $joueurs = $jeu->type == 'equipes' ? Admin::getEquipes($ligne['id']) : Admin::getListeJoueur($ligne['id']);
            $scores = $jeu->type_scores == 'scores' ? Admin::getListeScorePartie2($ligne['id']) : Admin::getListeClassement2($ligne['id']);
            Partie::modPartie($joueurs, $scores, $jeu, $ligne['id']);
        }
    }

}